import more_itertools
import itertools
import operator
import logging
import typing
from pathlib import Path

from .page import Page
from .feeds import RSSFeed


class Collection:
    """Collection objects serve as a way to quickly process pages that have a
    LARGE portion of content that is similar or file driven.

    The most common form of collection would be a Blog, but can also be
    static pages that have their content stored in a dedicated file.

    Currently, collections must come from a content_path and all be the same
    content type.


    Example
    -------
    from render_engine import Collection

    @site.register_collection()
    class BasicCollection(Collection):
        pass


    Attributes
    ----------
    engine: str, optional
        The engine that the collection will pass to each page. Site's default
        engine
    template: str
        The template that each page will use to render
    routes: List[str]
        all routes that the file should be created at. default []
    content_path: List[PathString], optional
        the filepath to load content from.
    includes: List[str], optional
        the types of files in the content path that will be processed
        default ["*.md", "*.html"]
    has_archive: Bool
        if `True`, create an archive page with all of the processed pages saved
        as `pages`. default `False`
    _archive_template: str, optional
        template filename that will be used if `has_archive==True` default: archive.html"
    _archive_slug: str, optional
        slug for rendered page if `has_archive == True` default: all_posts
    _archive_content_type: Type[Page], optional
        content_type for the rendered archive page
    _archive_reverse: Bool, optional
        should the sorted `pages` be listed in reverse order. default: False

    """
    engine = ''
    page_content_type: Page = Page
    content_path: str = "content"
    template: str = "page.html"
    includes: typing.List[str] = ["*.md", "*.html"]
    routes: typing.List[str] = [""]
    subcollections: typing.List[str] = []
    archive_template: str = "archive.html"
    archive_slug: str = "all_posts"
    archive_content_type: Page = Page
    archive_reverse: bool = False


    def __init__(
            self,
            *,
            title:str='',
            content_items: typing.List[typing.Optional[Page]]=[],
            has_archive: bool = False,
    ):
        self.content_items = content_items
        self.title = title
        self.has_archive = has_archive

    @staticmethod
    def archive_default_sort(cls):
        """attribute pulled from a rendered Page to sort `pages`"""
        return cls.slug

    @property
    def pages(self) -> typing.List[typing.Type[Page]]:
        """Iterate through set of pages and generate a `Page`-like object for each."""
        _pages = self.content_items

        if not Path(self.content_path).exists():
            return _pages # Do nothing if the path does not exist

        if Path(self.content_path).samefile('/'):
            logging.warning(f'{self.content_path=}! Accessing Root Directory is Dangerous...')

        for pattern in self.includes:

            for filepath in Path(self.content_path).glob(pattern):
                page = self.page_content_type(content_path=filepath)
                page.routes = self.routes
                page.template = self.template

                _pages.append(page)

        return _pages

    @property
    def archive(self):
        """Create a `Page` object for those items"""
        archive_page = self.archive_content_type(no_index=True)
        archive_page.template = self.archive_template
        archive_page.slug = self.archive_slug
        archive_page.engine = ""
        archive_page.title = self.__class__.__name__
        archive_page.pages = sorted(
            self.pages,
            key=lambda p: self.archive_default_sort(p),
            reverse=self.archive_reverse,
        )
        return archive_page

    @classmethod
    def from_subcollection(cls, collection, attr, attrval):
        content_items = []

        for page in collection.pages:

            if attrval in getattr(page, attr, []):
                content_items.append(page)


        return cls(title=attrval, content_items=content_items, has_archive=True)



"""A setuptools based setup module.
See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from os import path
# io.open is needed for projects that support Python 2.7
# It ensures open() defaults to text mode with universal newlines,
# and accepts an argument to specify the text encoding
# Python 3 only projects can skip this import
# from io import open

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

setup(
    name='render_engine',  # Required
    version='2020.6.2',  # Required
    description='Static Page Generation with Flask-like simplicity and flair ✨',  # Optional
    long_description=long_description,  # Optional
    long_description_content_type='text/markdown',  # Optional (see note above)
    url='https://github.com/kjaymiller/render_engine',  # Optional
    author='Jay Miller',  # Optional
    author_email='jay@productivityintech.com',  # Optional
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Text Processing :: Markup :: HTML',

        # Pick your license as you wish
        'License :: OSI Approved :: MIT License',

        # Natural Language
        'Natural Language :: English',

        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],

    # This field adds keywords for your project which will appear on the
    # project page. What does your project relate to?
    #
    # Note that this is a string of words separated by whitespace, not a list.
    keywords='static site web framework',  # Optional

    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),  # Required
    python_requires='>=3.7',
    install_requires=[
    'pendulum',
    'markdown',
    'jinja2',
    'more-itertools',
    'typer',
    ],  # Optional
    extras_require={  # Optional
        'dev': ['pipenv', 'check-manifest'],
        'test': ['pytest', 'pytest-cov', 'black'],
    },

    package_data={  # Optional
    #    'sample': ['package_data.dat'],
         'render_engine': ['rss/*', 'templates/*'],
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files
    #
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('my_data', ['data/data_file'])],  # Optional

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # `pip` to create the appropriate form of executable for the target
    # platform.
    #
    # For example, the following would provide a command called `sample` which
    # executes the function `main` from this package when invoked:
    # entry_points={  # Optional
    #    'console_scripts': [
    #        'sample=sample:main',
    #    ],
    #},

    # List additional URLs that are relevant to your project as a dict.
    #
    # This field corresponds to the "Project-URL" metadata fields:
    # https://packaging.python.org/specifications/core-metadata/#project-url-multiple-use
    #
    # Examples listed include a pattern for specifying where the package tracks
    # issues, where the source is hosted, where to say thanks to the package
    # maintainers, and where to support the project financially. The key is
    # what's used to render the link text on PyPI.
    project_urls={  # Optional
        'Documentation': 'https://render-engine.readthedocs.io/en/latest',
        'Bug Reports': 'https://github.com/kjaymiller/render_engine/issues',
        'Funding': 'https://paypal.me/kjaymiller',
        'Say Thanks!': 'http://saythanks.io/to/kjaymiller',
        'Source': 'https://github.com/kjaymiller/render_engine/',
    },
)



import logging
import json

class Search:
    def __init__(self, name):
        self.name = name

    def build_index(self, pages, keys, filepath='search.json'):
        search_index = []

        for page in pages:
            logging.debug(f'{page=}')

            if keys:
                search_index.append(
                        {x: str(y) for x,y in vars(page).items() if x in keys}
                        )

            else:
                logging.debug(vars(page))
                search_index.append({x: str(y) for x,y in vars(page).items()})

        with open(filepath, 'w') as json_file:
            return json.dump(search_index, json_file)


Fuse = Search('fuse')
